'use strict';

var postcss = require('postcss');
var postcssImport = require('postcss-import');
var postcssUrl = require('postcss-url');
var postcssModules = require('postcss-modules');
var cosmiconfig = require('cosmiconfig');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var postcss__default = /*#__PURE__*/_interopDefaultLegacy(postcss);
var postcssImport__default = /*#__PURE__*/_interopDefaultLegacy(postcssImport);
var postcssUrl__default = /*#__PURE__*/_interopDefaultLegacy(postcssUrl);
var postcssModules__default = /*#__PURE__*/_interopDefaultLegacy(postcssModules);

const htmlTags = ['a', 'abbr', 'address', 'area', 'article', 'aside', 'audio', 'b', 'base', 'bdi', 'bdo', 'blockquote', 'body', 'br', 'button', 'canvas', 'caption', 'cite', 'code', 'col', 'colgroup', 'data', 'datalist', 'dd', 'del', 'details', 'dfn', 'dialog', 'div', 'dl', 'dt', 'em', 'embed', 'fieldset', 'figcaption', 'figure', 'footer', 'form', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'head', 'header', 'hgroup', 'hr', 'html', 'i', 'iframe', 'img', 'input', 'ins', 'kbd', 'label', 'legend', 'li', 'link', 'main', 'map', 'mark', 'menu', 'menuitem', 'meta', 'meter', 'nav', 'noscript', 'object', 'ol', 'optgroup', 'option', 'output', 'p', 'param', 'picture', 'pre', 'progress', 'q', 'rp', 'rt', 'ruby', 's', 'samp', 'script', 'section', 'select', 'slot', 'small', 'source', 'span', 'strong', 'style', 'sub', 'summary', 'sup', 'svg', 'table', 'tbody', 'td', 'template', 'textarea', 'tfoot', 'th', 'thead', 'time', 'title', 'tr', 'track', 'u', 'ul', 'var', 'video', 'wbr'];
const htmlComponents = htmlTags.map(tag => tag[0].toUpperCase() + tag.slice(1));
/* 
May be in future...

export const svgTags = [
    'a',
    'altGlyph',
    'altGlyphDef',
    'altGlyphItem',
    'animate',
    'animateColor',
    'animateMotion',
    'animateTransform',
    'animation',
    'audio',
    'canvas',
    'circle',
    'clipPath',
    'color-profile',
    'cursor',
    'defs',
    'desc',
    'discard',
    'ellipse',
    'feBlend',
    'feColorMatrix',
    'feComponentTransfer',
    'feComposite',
    'feConvolveMatrix',
    'feDiffuseLighting',
    'feDisplacementMap',
    'feDistantLight',
    'feDropShadow',
    'feFlood',
    'feFuncA',
    'feFuncB',
    'feFuncG',
    'feFuncR',
    'feGaussianBlur',
    'feImage',
    'feMerge',
    'feMergeNode',
    'feMorphology',
    'feOffset',
    'fePointLight',
    'feSpecularLighting',
    'feSpotLight',
    'feTile',
    'feTurbulence',
    'filter',
    'font',
    'font-face',
    'font-face-format',
    'font-face-name',
    'font-face-src',
    'font-face-uri',
    'foreignObject',
    'g',
    'glyph',
    'glyphRef',
    'handler',
    'hkern',
    'iframe',
    'image',
    'line',
    'linearGradient',
    'listener',
    'marker',
    'mask',
    'metadata',
    'missing-glyph',
    'mpath',
    'path',
    'pattern',
    'polygon',
    'polyline',
    'prefetch',
    'radialGradient',
    'rect',
    'script',
    'set',
    'solidColor',
    'stop',
    'style',
    'svg',
    'switch',
    'symbol',
    'tbreak',
    'text',
    'textArea',
    'textPath',
    'title',
    'tref',
    'tspan',
    'unknown',
    'use',
    'video',
    'view',
    'vkern',
] as const
*/

const loadRawConfig = async loaderContext => {
  const explorer = cosmiconfig.cosmiconfig('postcss');

  try {
    const found = await explorer.search();

    if (found === null || found.isEmpty) {
      throw 'Config is empty';
    }

    const {
      config,
      filepath
    } = found;
    loaderContext.addBuildDependency(filepath);
    loaderContext.addDependency(filepath);

    if (typeof config === 'function') {
      const ctx = {
        env: loaderContext.mode,
        mode: loaderContext.mode,
        file: loaderContext.resourcePath,
        webpackLoaderContext: loaderContext,
        options: {}
      };
      return config(ctx);
    }

    return config;
  } catch (e) {
    return {};
  }
};

function* readPlugins(plugins) {
  if (Array.isArray(plugins)) {
    for (const item of plugins) {
      if (Array.isArray(item)) {
        const [plugin, options = {}] = item;
        yield [plugin, options];
      } else {
        const plugin = item;
        const options = {};
        yield [plugin, options];
      }
    }
  } else {
    for (const [plugin, val] of Object.entries(plugins)) {
      if (!!val) {
        const options = typeof val === 'object' ? val : {};
        yield [plugin, options];
      }
    }
  }
}

const ignorePlugins = ['postcss-import', 'postcss-url', 'postcss-modules'];

const preparePlugins = plugins => {
  const result = [];

  for (const [plugin, options] of readPlugins(plugins)) {
    if (!ignorePlugins.includes(plugin)) {
      const module = require(plugin);

      result.push(module(options));
    }
  }

  return result;
};

const prepareProcessOptions = (options, loaderContext) => {
  options.from = loaderContext.resourcePath;
  options.to = undefined;
  return options;
};

const loadConfig = async loaderContext => {
  const raw = await loadRawConfig(loaderContext);
  const plugins = preparePlugins(raw.plugins || []);
  const processOptions = prepareProcessOptions(raw.processOptions || {}, loaderContext);
  return {
    plugins,
    processOptions
  };
};

async function loader(source) {
  const {
    plugins,
    processOptions
  } = await loadConfig(this);
  const out = [`import { createComponent } from '@whatsup/webpack-loader-css-components/factory'`];
  const {
    css
  } = await postcss__default["default"]([postcssImport__default["default"]({
    // hack, used only for add dependency
    filter: path => {
      const absolutePath = this.utils.absolutify(this.context, path);
      this.addDependency(absolutePath);
      return true;
    }
  }), postcssUrl__default["default"]({
    url: async asset => {
      try {
        this.addDependency(asset.absolutePath);
        const module = await this.importModule(asset.absolutePath);
        return module.default;
      } catch (e) {
        return asset.originUrl;
      }
    }
  }), postcssModules__default["default"]({
    getJSON: (_, json) => {
      out.push(`const styles = ${JSON.stringify(json)}`);

      for (let i = 0; i < htmlTags.length; i++) {
        const tag = htmlTags[i];
        const component = htmlComponents[i];
        out.push(`export const ${component} = /*#__PURE__*/ createComponent('${tag}', styles)`);
      }
    }
  }), ...plugins]).process(source, processOptions);
  out.push(`const result = []`);
  out.push(`result.push([module.id, \`${css}\`])`);
  out.push(`result.locals = styles`);
  out.push(`export default result`);
  return out.join('\n');
}

module.exports = loader;
//# sourceMappingURL=index.js.map
